@@ -15,7 +15,7 @@
 import functools
 import testflows.settings as settings
 
-from testflows._core.flags import Flags, SKIP, LAST_RETRY
+from testflows._core.flags import Flags, SKIP, RETRY
 from testflows._core.testtype import TestType
 from testflows._core.message import Message
 from testflows._core.cli.colors import color
@@ -41,14 +41,18 @@ def color_result(result, attrs=None):
         raise ValueError(f"unknown result {result}")
 
 def add_result(msg, results):
+    flags = Flags(msg["test_flags"])
+    cflags = Flags(msg["test_cflags"])
     result = msg["result_type"]
-    if getattr(TestType, msg["test_type"]) < TestType.RetryIteration:
+
+    if getattr(TestType, msg["test_type"]) < TestType.Iteration:
         if not result.startswith("X"):
             return
-
-    flags = Flags(msg["test_flags"])
     if flags & SKIP and settings.show_skipped is False:
         return
+    if cflags & RETRY:
+        return
+
     if not result in ("OK", "Skip"):
         results[msg["test_id"]] = (msg, result)
 
@@ -74,16 +78,11 @@ def generate(results, divider, only_new=False):
             out += color(f" \u1405 {msg['result_reason']}", "white", attrs=["dim"])
         out += "\n"
 
-        if (getattr(TestType, msg["test_type"]) == TestType.RetryIteration
-                and not Flags(msg["test_flags"]) & LAST_RETRY):
-            if settings.show_retries:
-                retries += out
+        if result.startswith("X"):
+            if not only_new:
+                xfails += out
         else:
-            if result.startswith("X"):
-                if not only_new:
-                    xfails += out
-            else:
-                fails += out
+            fails += out
 
     if retries:
         retries = color(f"{divider}Retries\n\n", "white", attrs=["bold"]) + retries
