@@ -14,4 +14,6 @@
 # limitations under the License.
 from testflows._core.parallel import Context, ContextVar, copy_context
 from testflows._core.parallel.executor.thread import ThreadPoolExecutor as Pool, ThreadPoolExecutor as ThreadPool
-from testflows._core.parallel.executor.asyncio import AsyncPoolExecutor as AsyncPool
+from testflows._core.parallel.executor.thread import SharedThreadPoolExecutor as SharedPool, SharedThreadPoolExecutor as SharedThreadPool
+from testflows._core.parallel.executor.asyncio import AsyncPoolExecutor as AsyncPool, SharedAsyncPoolExecutor as SharedAsyncPool
+from testflows._core.parallel.executor.process import ProcessPoolExecutor as ProcessPool, SharedProcessPoolExecutor as SharedProcessPool, process_service
